#include "stdio.h"
#include "main.h"

/*
 * Main function
 * Simple example of EyePoint IVM analog signature analyzer usage
 */
int main(void)
{
    printf("EyePoint IVM application example\n");
    printf("--------------------------------\n");

    iv_curve_t IVCurve;
    device_t Device;

    Device = OpenDevice();
    
    if (Device != -1)
    {
        // Device successfully opened
        PrintDeviceInfo(Device);
        CalibrateDevice(Device);
        SetupDevice(Device);
        MakeMeasurement(Device, &IVCurve);
        CloseDevice(Device);
        SaveMeasurement(&IVCurve);
    }
    
    printf("Press any key to exit... \n");
    WAIT_FOR_ANY_KEY();
    return 0;
}

/*
 * Ask user for a serial port number
 * and try to open device on this port
 */
device_t OpenDevice(void)
{
    printf("Enter serial port.\n");
    printf("Format for Windows: com:\\\\.\\COM29\n");
    printf("Format for Linux: com:///dev/ttyACM0\n");

    char SerialPortURL[64];
	scanf("%s", SerialPortURL, (unsigned)ARRAY_LEN(SerialPortURL));

    device_t Device = ivm_open_device(SerialPortURL);
    if (Device == -1)
    {
        printf("Cannot open device.\n");
        printf("Please check serial port URL and try again.\n");
    }

    return Device;
}

/*
 * Read general product information
 * from the connected device
 */
void PrintDeviceInfo(device_t Device)
{
    result_t Result;
    printf("Read device information... ");

    ivm_get_identity_information_t DeviceInfo;
    Result = ivm_get_identity_information(Device, &DeviceInfo);
    CHECK_RESULT(Result, Device);

    ivm_get_device_rank_t DeviceRankInfo;
	Result = ivm_get_device_rank(Device, &DeviceRankInfo);
    CHECK_RESULT(Result, Device);

    printf("Done\n");

    printf("  -- Device information --\n");
    printf("  Product: %s %s\n", DeviceInfo.Manufacturer, DeviceInfo.ProductName);
    printf("  Hardware version: %d.%d.%d\n", DeviceInfo.HardwareMajor, DeviceInfo.HardwareMinor, DeviceInfo.HardwareBugfix);
    printf("  Serial number: %d\n", DeviceInfo.SerialNumber);
    printf("  Device rank: %d\n", DeviceRankInfo.Rank);
    printf("  Firmware version: %d.%d.%d\n", DeviceInfo.FirmwareMajor, DeviceInfo.FirmwareMinor, DeviceInfo.FirmwareBugfix);
}

/*
 * Run device automatic calibration
 * Normally should be done only once
 * after device power on before all measurements
 */
void CalibrateDevice(device_t Device)
{
    printf("Run device automatic calibration... ");
    result_t Result;
    ivm_start_autocalibration_t CalibrationOut;

    Result = ivm_start_autocalibration(Device, &CalibrationOut);
    CHECK_RESULT(Result, Device);

    printf("Done\n");

    if (CalibrationOut.Result == IVM_CALIBRATION_OK)
    {
        printf("  Device calibrated successfully\n");
    }
    else
    {
        printf("  There were some errors during calibration\n");
    }
}

/*
 * Setup device for measurements
 * Settings will be applied for all following measurements
 * until the next reconfiguration or device power down or reset.
 */
void SetupDevice(device_t Device)
{
    result_t Result;
    printf("Setup device... ");

    // 1. Fill settings structure
    ivm_measurement_settings_t MeasurementSettings;
    
    MeasurementSettings.ProbeSignalFrequency = 100;          // Probe signal frequency in Hz
    MeasurementSettings.NumberPoints = NUM_POINTS_IN_CURVE;  // We want to get only one period (Resolution: NUM_POINTS_IN_CURVE / period)
    MeasurementSettings.SamplingRate = MeasurementSettings.ProbeSignalFrequency * NUM_POINTS_IN_CURVE;
    MeasurementSettings.NumberChargePoints = 0;              // No precharge
    MeasurementSettings.MaxVoltage = 12;                     // This is open circuit voltage. Real voltage will be smaller.
    MeasurementSettings.CurrentSensorMode = IVM_CURRENT_SENSE_MODE_I_MID; // Medium currents
    MeasurementSettings.OutputMode = IVM_OUT_MODE_PROBE_SIGNAL_CONTINUOUS;
    
    // 2. Apply settings
    Result = ivm_set_measurement_settings(Device, &MeasurementSettings);
    CHECK_RESULT(Result, Device);
    
    printf("Done\n");
}

/*
 * Launch measuremnt,
 * wait until the measurement will be finished,
 * read measurement results from the device
 */
void MakeMeasurement(device_t Device, iv_curve_t *IVCurve)
{
    result_t Result;
    printf("Make measuremnet... ");

    // 1. Launch measuremnt.
    Result = ivm_start_measurement(Device);
    CHECK_RESULT(Result, Device);


    // 2. Wait until the measurement will be finished
    // ivm_start_measurement() is non-blocking
    // check, that the measurement is finished,
    // before reading results is required
    ivm_check_measurement_status_t MeasurementStatus;
    do
    {
        SLEEP_MS(100); // Small delay
        Result = ivm_check_measurement_status(Device, &MeasurementStatus);
        CHECK_RESULT(Result, Device);

    } while (MeasurementStatus.ReadyStatus != IVM_MEASUREMENT_COMPLETE);


    // 3. Read measurement results from the device
    int FrameNumber, PointIdx;
    ivm_in_get_measurement_t MeasurementIn;
    ivm_out_get_measurement_t MeasurementOut;
    
    // Read full frames
    for (FrameNumber = 0; FrameNumber < NUM_POINTS_IN_CURVE / IVM_FRAME_SIZE; FrameNumber++)
    {
        MeasurementIn.FrameNumber = FrameNumber;
        
        Result = ivm_get_measurement(Device, &MeasurementIn, &MeasurementOut);
        CHECK_RESULT(Result, Device);

        // Copy frame
        for (PointIdx = FrameNumber * IVM_FRAME_SIZE;
            PointIdx < (FrameNumber + 1) * IVM_FRAME_SIZE;
            PointIdx++)
        {
            IVCurve->Voltage[PointIdx] = MeasurementOut.Voltage[PointIdx % IVM_FRAME_SIZE];
            IVCurve->Current[PointIdx] = MeasurementOut.Current[PointIdx % IVM_FRAME_SIZE];
        }
    }

    // Read residual frame
    if (NUM_POINTS_IN_CURVE % IVM_FRAME_SIZE > 0)
    {
        MeasurementIn.FrameNumber = FrameNumber; // After for loop FrameNumber should be correct
        
        Result = ivm_get_measurement(Device, &MeasurementIn, &MeasurementOut);
        CHECK_RESULT(Result, Device);

        // Copy valid part of the last frame
        for (PointIdx = FrameNumber * IVM_FRAME_SIZE;
            PointIdx < NUM_POINTS_IN_CURVE;
            PointIdx++)
        {
            IVCurve->Voltage[PointIdx] = MeasurementOut.Voltage[PointIdx % IVM_FRAME_SIZE];
            IVCurve->Current[PointIdx] = MeasurementOut.Current[PointIdx % IVM_FRAME_SIZE];
        }
    }

    printf("Done\n");
}


/*
 * Save measurede curve to results.json file
 */
void SaveMeasurement(iv_curve_t *IVCurve)
{
    printf("Save measurement results to file results.json... ");

    int PointIdx;

    FILE *ResultFile = fopen("results.json", "w");

    fprintf(ResultFile, "{\n");

    // Save voltages
    fprintf(ResultFile, "    \"voltage\": [");
    for (PointIdx = 0; PointIdx < NUM_POINTS_IN_CURVE - 1; PointIdx++)
    {
        fprintf(ResultFile, "%f, ", IVCurve->Voltage[PointIdx]);
    }
    fprintf(ResultFile, "%f],\n", IVCurve->Voltage[NUM_POINTS_IN_CURVE - 1]);

    
    // Save currents
    fprintf(ResultFile, "    \"current\": [");
    for (PointIdx = 0; PointIdx < NUM_POINTS_IN_CURVE - 1; PointIdx++)
    {
        fprintf(ResultFile, "%f, ", IVCurve->Current[PointIdx]);
    }
    fprintf(ResultFile, "%f],\n", IVCurve->Current[NUM_POINTS_IN_CURVE - 1]);

    fprintf(ResultFile, "    \"voltage_units\": \"Volts\",\n");
    fprintf(ResultFile, "    \"current_units\": \"mA\"\n");
    fprintf(ResultFile, "}\n");

    fclose(ResultFile);

    printf("Done\n");
}

void CloseDevice(device_t Device)
{
    printf("Close device... ");
    result_t Result;
    
    Result = ivm_close_device(&Device);
    CHECK_RESULT(Result, Device);

    printf("Done\n");
}
